import 'dart:convert';

class Clock {
  final DateTime date;
  final DateTime scheduledStart;
  final DateTime scheduledEnd;
  final Duration scheduledBreak;
  final DateTime actualStart;
  final DateTime actualEnd;
  final Duration actualBreak;
  final String notes;

  Clock({
    this.date,
    this.scheduledStart,
    this.scheduledEnd,
    this.scheduledBreak,
    this.actualStart,
    this.actualEnd,
    this.actualBreak,
    this.notes
  });

  factory Clock.fromMap(Map<String, dynamic> map) => Clock(
    date: map['date'],
    scheduledStart: map['scheduledStart'],
    scheduledEnd: map['scheduledEnd'],
    scheduledBreak: map['scheduledBreak'],
    actualStart: map['actualStart'],
    actualEnd: map['actualEnd'],
    actualBreak: map['actualBreak'],
    notes: map['notes']
  );

  Map<String, dynamic> toMap() => {
    'date': date,
    'scheduledStart': scheduledStart,
    'scheduledEnd': scheduledEnd,
    'scheduledBreak': scheduledBreak,
    'actualStart': actualStart,
    'actualEnd': actualEnd,
    'actualBreak': actualBreak,
    'notes': notes
  };

}

class Task {
  final DateTime taskStart;
  final DateTime taskEnd;
  final String job;

  Task({
    this.taskStart,
    this.taskEnd,
    this.job,
  });

  factory Task.fromList(List dataList) => Task(
        // Dart doesn't parse just times so include a zero date to allow parsing
        taskStart: DateTime.parse("0000-00-00T" + dataList[0]),
        taskEnd: DateTime.parse("0000-00-00T" + dataList[1]),
        job: dataList[2],
      );

  Map<String, dynamic> toMap() =>
      {"taskStart": taskStart, "taskEnd": taskEnd, "job": job};
}

class Shift {
  final int assignedTo;
  final String assignStatus;
  final String shiftUpdateType;
  final int startDate;
  final int startDateSkey;
  final int unitSkey;
  final int genShiftId;
  final int shiftSeqNo;
  final int startTime;
  final int duration;
  final String staffGroupId;
  final String shiftTaskList;
  final String lastUpdateType;
  final int preassignedTo;
  final int empStartDate;
  final int empStartTime;
  final int empDuration;
  final int empBreakDuration;
  final String errorCd;
  final double shiftCost;
  final double empCost;
  final int breakDuration;
  final String shiftLock;
  final double seasonalityAvg;
  final int lastUpdateTime;
  final String lastUser;

  Shift({
    this.assignedTo,
    this.assignStatus,
    this.shiftUpdateType,
    this.startDate,
    this.startDateSkey,
    this.unitSkey,
    this.genShiftId,
    this.shiftSeqNo,
    this.startTime,
    this.duration,
    this.staffGroupId,
    this.shiftTaskList,
    this.lastUpdateType,
    this.preassignedTo,
    this.empStartDate,
    this.empStartTime,
    this.empDuration,
    this.empBreakDuration,
    this.errorCd,
    this.shiftCost,
    this.empCost,
    this.breakDuration,
    this.shiftLock,
    this.seasonalityAvg,
    this.lastUpdateTime,
    this.lastUser,
  });

  factory Shift.fromMap(Map<String, dynamic> json) => Shift(
        assignedTo: json["assignedTo"],
        assignStatus: json["assignStatus"],
        shiftUpdateType: json["shiftUpdateType"],
        startDate: json["startDate"],
        startDateSkey: json["startDateSkey"],
        unitSkey: json["unitSkey"],
        genShiftId: json["genShiftId"],
        shiftSeqNo: json["shiftSeqNo"],
        startTime: json["startTime"],
        duration: json["duration"],
        staffGroupId: json["staffGroupId"],
        shiftTaskList: json["shiftTaskList"],
        lastUpdateType: json["lastUpdateType"],
        preassignedTo: json["preassignedTo"],
        empStartDate: json["empStartDate"],
        empStartTime: json["empStartTime"],
        empDuration: json["empDuration"],
        empBreakDuration: json["empBreakDuration"],
        errorCd: json["errorCd"],
        shiftCost: json["shiftCost"],
        empCost: json["empCost"],
        breakDuration: json["breakDuration"],
        shiftLock: json["shiftLock"],
        seasonalityAvg: json["seasonalityAvg"],
        lastUpdateTime: json["lastUpdateTime"],
        lastUser: json["lastUser"],
      );

  Map<String, dynamic> toMap() => {
        "assignedTo": assignedTo,
        "assignStatus": assignStatus,
        "shiftUpdateType": shiftUpdateType,
        "startDate": startDate,
        "startDateSkey": startDateSkey,
        "unitSkey": unitSkey,
        "genShiftId": genShiftId,
        "shiftSeqNo": shiftSeqNo,
        "startTime": startTime,
        "duration": duration,
        "staffGroupId": staffGroupId,
        "shiftTaskList": shiftTaskList,
        "lastUpdateType": lastUpdateType,
        "preassignedTo": preassignedTo,
        "empStartDate": empStartDate,
        "empStartTime": empStartTime,
        "empDuration": empDuration,
        "empBreakDuration": empBreakDuration,
        "errorCd": errorCd,
        "shiftCost": shiftCost,
        "empCost": empCost,
        "breakDuration": breakDuration,
        "shiftLock": shiftLock,
        "seasonalityAvg": seasonalityAvg,
        "lastUpdateTime": lastUpdateTime,
        "lastUser": lastUser
      };
}

class PersonWeek {
  final int personId;
  final DateTime weekStartDate;
  final DateTime weekEndDate;
  final EmployeeData employee;
  final String employeeType;
  final String employeeGroup;
  final List<Object> shifts;
  final String holidayDates;
  final String unitId;
  final double minNormalHours;
  final double maxNormalHours;
  final double originalMinNormalHours;
  final String hourlyOrSalaried;
  final String homeStore; // TODO: Make bool
  final int minNormalMins;
  final int totalScheduledMins;
  final double minNormalHoursforStoreType;
  final int maxNormalMins;
  final int originalMinNormalMins;
  final int minNormalMinsforStoreType;
  final double totalScheduleHours;
  final bool hourlyPaidEmployee;
  final bool employeeOnContract;
  final bool onLeaveInThisWeek;

  PersonWeek(
      {this.personId,
      this.weekStartDate,
      this.weekEndDate,
      this.employee,
      this.employeeType,
      this.employeeGroup,
      this.shifts,
      this.holidayDates,
      this.unitId,
      this.minNormalHours,
      this.maxNormalHours,
      this.originalMinNormalHours,
      this.hourlyOrSalaried,
      this.homeStore,
      this.minNormalMins,
      this.totalScheduledMins,
      this.minNormalHoursforStoreType,
      this.maxNormalMins,
      this.originalMinNormalMins,
      this.minNormalMinsforStoreType,
      this.totalScheduleHours,
      this.hourlyPaidEmployee,
      this.employeeOnContract,
      this.onLeaveInThisWeek});

  factory PersonWeek.fromMap(Map<String, dynamic> weekData) {
    List<Shift> shiftList = [];
    if (weekData["shifts"] != null) {
      if (!(weekData["shifts"] is List)) {
        var shiftsString;
        String a = weekData["shifts"];
        shiftsString = weekData["shifts"].split("}, {");
        var shifts = [];

        int i = 0;
        String stringMap;
        for (String shift in shiftsString) {
          String formattedShift = shift.replaceAll('[', '').replaceAll('{', '').replaceAll('}', '').replaceAll(']', '');
          stringMap = "{$formattedShift}";
          shifts.add(json.decode(stringMap));
          i++;
        }
        shifts.forEach((shift) {
          shiftList.add(Shift.fromMap(shift));
        });

      } else {
        weekData["shifts"].forEach((shift) {
          shiftList.add(Shift.fromMap(shift));
        });      
      }
    }

    var employeeData;
    var employeeMap;
    if (weekData["employee"] is String) {
      var employeeMap = json.decode(weekData["employee"]);
      employeeData = EmployeeData.fromMap(employeeMap);
    } else {
      employeeMap = weekData["employee"];
      employeeData = EmployeeData.fromMap(employeeMap);
    }

    hourlyPaidEmployee: weekData["hourlyPaidEmployee"];
    employeeOnContract: weekData["employeeOnContract"];
    onLeaveInThisWeek: weekData["onLeaveInThisWeek"];

    bool hourlyPaidEmployeeParse;
    bool employeeOnContractParse;
    bool onLeaveInThisWeekParse;

    if (weekData["hourlyPaidEmployee"] is bool){
      hourlyPaidEmployeeParse = weekData["hourlyPaidEmployee"];
    } else {
      if (weekData["hourlyPaidEmployee"] == 1) {
        hourlyPaidEmployeeParse = true;
      } else {
        hourlyPaidEmployeeParse = false;
      }
    }

    if (weekData["employeeOnContract"] is bool){
      employeeOnContractParse = weekData["employeeOnContract"];
    } else {
      if (weekData["empoyeeOnContract"] == 1) {
        employeeOnContractParse = true;
      } else {
        employeeOnContractParse = false;
      }
    }

    if (weekData["onLeaveInThisWeek"] is bool){
      onLeaveInThisWeekParse = weekData["onLeaveInThisWeek"];
    } else {
      if (weekData["onLeaveInThisWeek"] == 1) {
        onLeaveInThisWeekParse = true;
      } else {
        onLeaveInThisWeekParse = false;
      }
    }

    return PersonWeek(
      personId: employeeData.personId,
      weekStartDate:
          DateTime.fromMillisecondsSinceEpoch(weekData["weekStartDate"]),
      weekEndDate: DateTime.fromMillisecondsSinceEpoch(weekData["weekEndDate"]),
      employee: EmployeeData(
          employeeId: employeeData.employeeId,
          personId: employeeData.personId,
          firstName: employeeData.firstName,
          middleName: employeeData.middleName,
          lastName: employeeData.lastName),
      employeeType: weekData["employeeType"],
      employeeGroup: weekData["emplyeeGroup"],
      shifts: shiftList,
      holidayDates: weekData["holidayDates"],
      unitId: weekData["unitId"],
      minNormalHours: weekData["minNormalHours"],
      maxNormalHours: weekData["maxNormalHours"],
      originalMinNormalHours: weekData["originalMinNormalHours"],
      hourlyOrSalaried: weekData["hourlyOrSalaried"],
      homeStore: weekData["homeStore"],
      minNormalMins: weekData["minNormalMins"],
      totalScheduledMins: weekData["totalScheduledMins"],
      minNormalHoursforStoreType: weekData["minNormalHoursforStoreType"],
      maxNormalMins: weekData["maxNormalMins"],
      originalMinNormalMins: weekData["originalMinNormalMins"],
      minNormalMinsforStoreType: weekData["minNormalMinsforStoreType"],
      totalScheduleHours: weekData["totalScheduleHours"],
      hourlyPaidEmployee: hourlyPaidEmployeeParse,
      employeeOnContract: employeeOnContractParse,
      onLeaveInThisWeek: onLeaveInThisWeekParse,
    );
  }

  Map<String, dynamic> toMap() {
  
    List<String> shiftList = [];

    for (Shift shift in shifts) {
      shiftList.add(jsonEncode(shift.toMap()));
    }

    String shiftListString = "${shiftList}";

    String employeeString = json.encode({
        "employeeId": employee.employeeId,
        "personId": employee.personId,
        "firstName": employee.firstName,
        "middleName": employee.middleName,
        "lastName": employee.lastName
      });

    return {
      "personId": personId,
      "weekStartDate": weekStartDate.millisecondsSinceEpoch,
      "weekEndDate": weekEndDate.millisecondsSinceEpoch,
      "employee": employeeString,
      "employeeType": employeeType,
      "employeeGroup": employeeGroup,
      "shifts": shiftListString,
      "holidayDates": holidayDates,
      "unitId": unitId,
      "minNormalHours": minNormalHours,
      "maxNormalHours": maxNormalHours,
      "originalMinNormalHours": originalMinNormalHours,
      "hourlyOrSalaried": hourlyOrSalaried,
      "homeStore": homeStore,
      "minNormalMins": minNormalMins,
      "totalScheduledMins": totalScheduledMins,
      "minNormalHoursforStoreType": minNormalHoursforStoreType,
      "maxNormalMins": maxNormalMins,
      "originalMinNormalMins": originalMinNormalMins,
      "minNormalMinsforStoreType": minNormalMinsforStoreType,
      "totalScheduleHours": totalScheduleHours,
      "hourlyPaidEmployee": hourlyPaidEmployee,
      "employeeOnContract": employeeOnContract,
      "onLeaveInThisWeek": onLeaveInThisWeek,
    };
  }
}

class EmployeeData {
  final int personId;
  final String employeeId;
  final String firstName;
  final String middleName;
  final String lastName;

  EmployeeData(
      {this.personId,
      this.employeeId,
      this.firstName,
      this.middleName,
      this.lastName});

  factory EmployeeData.fromMap(Map<String, dynamic> json) => EmployeeData(
      personId: json["personId"],
      employeeId: json["employeeId"],
      firstName: json["firstName"],
      middleName: json["middleName"],
      lastName: json["lastName"]);
}
